<ul style="line-height: 13px; text-align: left; margin: 0; padding: 10px;">
    {% for key, value in report_details.items %}
        <li><strong>{{ key }}: </strong> {{ value | safe }}</li>
    {% endfor %}
</ul>
