from .compliance_form import (
    ComplianceFormSectionRuleTitleConfig,
    ComplianceFormSectionTitleConfig,
    ComplianceFormSignatureSectionRuleTitleConfig,
    ComplianceFormSignatureTitleConfig,
    ComplianceFormTitleConfig,
)
from .compliance_task import (
    ComplianceTaskComplianceTaskGroupTitleConfig,
    ComplianceTaskGroupTitleConfig,
    ComplianceTaskInstanceTitleConfig,
    ComplianceTaskMatrixPandasTitleConfig,
    ComplianceTaskTitleConfig,
    ReviewComplianceTaskTitleConfig,
)
from .compliance_type import ComplianceTypeTitleConfig
from .risk_managment import *
