# Generated by Django 4.2.9 on 2024-02-20 10:08

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbcompliance", "0011_alter_riskrule_parameters"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="compliancetype",
            options={
                "permissions": [("administrate_compliance", "Can Administrate Compliance")],
                "verbose_name": "Compliance Type",
                "verbose_name_plural": "Compliance Types",
            },
        ),
    ]
